;*****************************************************************************
;*									     *
;*			   UniPascal, version 1.50			     *
;*			  assembly part of CRT Unit			     *
;*									     *
;*		   (c) 1990 Software R&D Laboratory, Sofia		     *
;*									     *
;*****************************************************************************

                .define SPC

EquipmentBYTE	=	$01		; Equipment Byte
EquipKernel	=	$80		;   UniDOS kernel present
EquipGraph	=	$40		;   Graphics BIOS
EquipBASIC	=	$20		;   BASIC interpreter present
EquipYcode	=	$10		;   Ycode interpreter present
Equip_2Mhz	=	$08		;   Working on 2 Mhz
EquipShell	=	$01		;   UniDOS Shell present

                .ifdef  SPC
PASCAL_work     =       $da00
_UniCRT         =       $b8

UniCRT_acrsr    =       1
UniCRT_pcrsr    =       2
UniCRT_tmode    =       3
                .else
IOarea		=       $e600
BIOS_work       =       $ed00
PASCAL_work     =       $be00

CRTaddr         equ     IOarea + $00    ;607
CRTdata         equ     IOarea + $01

CRSRstart       =       BIOS_work + $1d
CRSRend         =       BIOS_work + $1e
CRSRblink       =       BIOS_work + $1f
CONparms        =       BIOS_work + $22 ; pointer to console parameters
                .endif

Interpreter     =       PASCAL_work+$fe ; return to Interpreter

MODEL		=	$fff7		;  $0 - 601/601U
					; $80 - 601A
					; $c0 - 601M
					; $e0 - 601T

                section PASCAL_work+$a0
Nentries	ds	1		; Nentries for CRT starts
ATTRIBUTE	ds	1		; Attribute for PLDIN 601A
                .ifdef  SPC
SAVEparms	ds	2
		ds	4		; unused now
NEWparms	ds	0		; start of console parameters
PauseKey	ds	1		; pause key set to none ($ff)
BreakKey	ds	1		; break key set to none ($ff)
OVRcrsr         ds      2               ; Active OVeRwrite Cursor Scan Lines
INScrsr         ds      2               ; Active Insert Cursor Scan Lines
PSVcrsr         ds      2               ; Passive cursor Scan Lines
                .endif
		ends

WINspaceAddr    =       PASCAL_work + $b0

                section WINspaceAddr
UniWINcomm      ds      2               ; UniWIN communication routine
                ends

		section $b0
x0		ds	2
x1		ds	2
a		ds	1
		ends

;------------------------------------------------------------------------------
TextMode	proc			; procedure TextMode(mode: ShortCard);
					; bw40 = 0
					; bw80 = 4
					; co40 = 5
		dw	0		; begin
		ins			; Pop return address
		ins			;
		pula			; Pop mode
		ins			; Pop MSByte
                .ifdef  SPC
                 ldb    #UniCRT_tmode
                 int    _UniCRT
                .else
                 clrb
                 tsta                   ; bw40 ?
                 beq     ModeOk         ; Y - set this mode

                 cmpa    #$4            ; bw80
                 blt     DoNothing      ; >= 4 - check for 601A

                 ldab    MODEL          ; Check machine for mode bw80
                 bmi     M601A          ; Machine is 601A.
                 clra                   ; Machine is 601. Only bw40 mode.
M601A
                 clrb
                 cmpa    #$5            ; Co40 ?
                 bne     ModeOk         ; No - go to set BW40 mode
                                        ; else store attribute to B
                 ldab    #$70           ; Text color = WHITE
                 ldaa    #4             ; Background = BLACK
ModeOk
                 int     $12            ; Initialize video & ClrScr
                 stab    ATTRIBUTE      ; Store ATTRIBUTE in reserved area
                .endif
DoNothing       jmp     Interpreter
		endp ;TextMode
;------------------------------------------------------------------------------
GotoXY		proc			; procedure GotoXY(X, Y: ShortCard);
		dw	0

		ins			; pop return address
		ins
		pulb			; Y position
		ins			; pop MSByte
		pula			; X position
		ins			; pop MSByte
		deca			; Pascal screen starts from 1, 1
		decb			; BIOS screen starts from 0, 0
		int	$15		; BIOS interrupt
		jmp	Interpreter	; return to Interpreter
		endp;	GotoXY
;------------------------------------------------------------------------------
ClrEOL		proc			; procedure ClrEol;
		dw	0		; begin
		ldaa	#24		; ch:= #24;
                bra     SendCharRTS     ; Send Char to screen and return
		endp; ClrEOL
;------------------------------------------------------------------------------
ClrScr		proc			; procedure ClrScr;
		dw	0		; begin
		ldaa	#12		; ch:= #12
SendCharRTS	=	*
		int	$22		; write(message, ch);
		rts			; end { ClrScr };
		endp; ClrScr
;------------------------------------------------------------------------------
ClrEOS		proc			; procedure ClrEoS;
		dw	0		; begin
		int	$14		; Get X&Y position
		sta	x0		; in B is Y position
		stb	x0+1		; X isn't interesting
                int     $6d             ;  UniWIN
                stb     patch+1
		bra	clreol		; ch:= #24;
LoopEOS
		incb
patch           cmpb    #25             ; BIOS screen coordinates:
					; X : 0..39 (0..79)
					; Y : 0..24
		beq	RetEOS
		clra			; X := 0
		int	$15		; GotoXY(0, B)
clr2eol
		ldaa	#24
		int	$22		; ClrEOL
		bra	LoopEOS
RetEOS
		lda	x0
		ldb	x0+1
		int	$15		; GotoXY(0, 0)
		rts
		endp; ClrEOS
;------------------------------------------------------------------------------
Delay		proc			; procedure Delay(MS: word);
		dw	0

		ins
		ins
		pla			; swap it
		plb
		pha
		phb
		tsx
		ldx	x, 0
		beq	rts
                ldb     #99
                lda     EquipmentBYTE
                bita    #Equip_2Mhz
                beq     loop
                ldb     #99*2+1
loop
                tba             ;                                    2|1sec
wait            deca            ;  2|1sec \                        
                nop             ;  2|1sec  \__10sec * 99= 990sec 
                nop             ;  2|1sec  /   5sec *199= 995sec 
                bne     wait    ;  4|1sec /                        
                dex             ;                                    4|2sec
                bne     loop    ;                                    4|2sec
rts                             ;      total loop counts 1000sec = 1 ms
		ins
		ins
		jmp	Interpreter
		endp; Delay
;------------------------------------------------------------------------------
PlayNote	proc			; procedure PlayNote(Note, Tembr: byte;
		dw	0		;		     length: word);
		ins			; pop return address
		ins

		pula			; length
		pulb
		sta	x0+1
		stb	x0
		plb
		ins
		pla
		ins
		ldx	x0
		int	$6f
		jmp	Interpreter
		endp; PlayNote
;------------------------------------------------------------------------------
PlayMelo	proc			; procedure PlayMelody(var Melody);
		dw	0
		ins
		ins
		tsx
		ldx	x
		ins
		ins
		int	$6e
exit		jmp	Interpreter
		endp; PlayMelody
;------------------------------------------------------------------------------
ActvCrsr	proc			; procedure ActvCrsr(sz: 0..4);
		dw	0
		ins
		ins
		pla
		anda	#$7		;  0  4
		ins
                ldx     UniWINcomm
                beq     NoUniWIN
                jsr     x, 2            ; Set Active Cursor
                bra     rts
NoUniWIN        .ifdef  SPC
                 ldb    #UniCRT_acrsr
                 int    _UniCRT
                .else
                 asla
                 beq     CursorOFF
                 tab
                 subb    #2
                 suba    #8
                 nega
                 bra     StoreCursor
CursorOFF        lda     #8             ; turn two Active cursors (INS/REP) OFF
                 tab
StoreCursor      sta     OVRcrsr
                 sta     CRSRstart
                 stb     INScrsr
                 lda     #7
                 sta     OVRcrsr+1
                 sta     INScrsr+1
                 sta     CRSRend
                .endif
rts             jmp     Interpreter
		endp; ActvCrsr
;------------------------------------------------------------------------------
PassCrsr	proc			; procedure PasvCrsr(sz: 0..4);
		dw	0
		ins
		ins
		pla
                .ifNdef SPC
                 tab                     ; ?
                 ldb    CRSRblink
                .endif
                anda    #$07
		ins
                ldx     UniWINcomm
                beq     NoUniWIN
                jsr     x, 4            ; Set Passive Cursor
                bra     rts
NoUniWIN
                .ifdef SPC
                 ldb    #UniCRT_pcrsr
                 int    _UniCRT
rts              jmp     Interpreter
                .else
                 asla
                 suba    #8
                 nega
                 aba                     ; addblinking
                 sta     PSVcrsr
;                sta     CRSRstart       ;        
                 ldb     #10
                 bsr     set6845
                 incb
                 lda     #7
                 sta     PSVcrsr+1
;                sta     CRSRend
                 bsr     set6845
rts              jmp     Interpreter

set6845          stb     CRTaddr
                 sta     CRTdata
                 rts
                .endif
		endp; PassCrsr
;------------------------------------------------------------------------------
SetColor	proc	; procedure SetColor(Attribute: ShortCard);
		dw	0
		ins			; pop return address
		ins
		pulb			; get parameter
		ins			; pop MSByte
		stb	ATTRIBUTE
		int	$16		; call BIOS
		jmp	Interpreter
		endp; SetColor
;------------------------------------------------------------------------------
GetColor	proc	; function GetColor : ShortCard;
		dw	0
                ldx     UniWINcomm
                beq     NoUniWIN
                jsr     x, 0            ; GetColor from uniwin
                bra     fix_X_0A
NoUniWIN        lda     ATTRIBUTE
                bra     fix_X_0A
                endp; GetColor
;------------------------------------------------------------------------------
WhereX		proc			; function WhereX: ShortCard;
		dw	0
		int	$14		; WhereXY
                inca                    ; X/Y position for BIOS starts from 0
                bra     fix_X_0A
		endp; WhereX
;------------------------------------------------------------------------------
WhereY		proc			; function WhereY: ShortCard;
		dw	0
		int	$14		; WhereXY
                incb
                bra     fix_X_0B
		endp; WhereY
;------------------------------------------------------------------------------
ScrXsize	proc			; function ScrXsize: byte;
		dw	0		; returns Screen X size
		int	$6d		; get screen size
fix_X_0A        =       *
		tsx
		clr	x, 1
		sta	x, 0
		jmp	Interpreter
		endp; ScrXsize
;------------------------------------------------------------------------------
ScrYsize	proc			; function ScrYsize: byte;
		dw	0		; returns Screen Y size
		int	$6d		; get screen size
fix_X_0B        =       *
                tba
                bra     fix_X_0A
		endp; ScrXsize
;------------------------------------------------------------------------------
GetMode         proc                    ; function GetMode: shortcard
                dw      0
                int     $13
                tstb
                beq     OK
                tsta
                bne     OK
                lda     #5              ; CO40
OK              bra     fix_X_0A
                endp; GetMode
;------------------------------------------------------------------------------
KeyPress	proc			; function KeyPressed: boolean;
		dw	0

		int	$10
		inca
		beq	NoKey
		ldaa	#1
NoKey           bra     fix_X_0A
		endp;	KeyPress
;------------------------------------------------------------------------------
ReadKey 	proc			; function ReadKey: char;
		dw	0
		int	$20
                bra     fix_X_0A
		endp;	ReadKey
;------------------------------------------------------------------------------
InitCRT 	proc
		dw	0
		lda	Nentries
		bne	AlreadyHere
                .ifNdef  SPC
                 ldx     CONparms
                 stx     SAVEparms
                 stx     x0
                 ldx     #NEWparms
                 stx     x1
                 stx     CONparms
                 ldx     #x0
                 lda     #8
                 clrb
                 int     $2d
                 ldx     #$ffff
                 stx     PauseKey
                .endif
AlreadyHere
		inc	Nentries
		rts
		endp; InitCRT
;------------------------------------------------------------------------------
FinitCRT	proc
		dw	0
		dec	Nentries
		bne	rts
                .ifNdef SPC
                 ldx     SAVEparms
                 stx     CONparms
                .endif
;                lda     x, 6            ; restore PassiveCursor
;                ldb     x, 7
;                sta     CrsrStart
;                stb     CrsrEnd
rts		rts
		endp; FinitCRT
;------------------------------------------------------------------------------

		public	ReadKey, Delay, PlayNote, KeyPress, PlayMelo
		public	ClrEOL, ClrEOS, ClrScr, SetColor, GetColor
                public  GotoXY, WhereX, WhereY, TextMode, GetMode
                public  PassCrsr, ActvCrsr, sCRxSIZE, scrYsize
		public	InitCRT, FinitCRT

		end.